****************************************************************************
*  UNIQID.PRG  - Unique ID / Filename generator.   v1.36
*                Returns a unique 8 character ID string, which can be
*                 used as an ID code, or as a filename.  The code that
*                 is returned is guaranteed to be a valid DOS filename,
*                 and to be unique for 2,821,109,907,456 calls.
*                UNIQID.PRG is network aware.
*                Written by Cory A. Jaeger
*       SYNTAX -  PROGRESS(tagname)
*
*
* FREE!!! - I wrote this program for my own use and am releasing
*           it into puplic domain on the following conditions.
*         - If you use it, PLEASE let me know.  (It's nice to know that
*           others find your work useful.)  CIS - 73424,536.
*         - A thank-you gift of $1 would be appreciated if you find that
*           you are frequently using this code either un-modified or
*           as a base for other code.  (HOWEVER, IT IS NOT REQUIRED!)
*         - I must get credit for this code if used in any publication.
*         - My name must not be removed from the comments of this code
*           as the original writer.
*         - If you release and updated/revised version, you must add
*           to these comments.  Include your name and a summary
*           of what you have fixed/added.  (Please, send me a copy of
*           the updated code.)
*
*
*  Parameters:    m.tag - 10 Character ID record identifier. [optional]
*                       - Allows the user to keep several seperate
*                         lists of ID#s/filenames.  (I recommend always
*                         excuding the tag name when you are getting file
*                         names.)
*
*               CAUTION - Although each tag name you use will return almost
*                         3 trillion unique ID's/filenames, the ID returned
*                         may not be unique between tags.
*                     ie) UNIQID() will never return the same ID twice,
*                         neither will UNIQID("ACCTCD"), however,
*                         the ID returned by UNIQID() and the ID
*                         returned by UNIQID("ACCTCD") could be the same!
*
***************************************************************************
* A total of 2,821,109,907,456 unique filenames/ID #'s will be returned
* before repetition begins.  (Return value is serialized, NOT random.)
PARAMETERS m.tag
PRIVATE m.oldrepro, st, oldsel, m.ret
IF EMPTY(m.tag) OR TYPE("m.tag")<>"C"
   m.tag="DEFAUL    "                            && IF invalid or no parameter passed, default to "DEFAUL    "
ELSE
   m.tag=PADR(UPPER(LEFT(LTRIM(m.tag),10)),10)   && Make sure tag name is exactly 6 upper case characters.
ENDIF
m.oldrepro=SET("REPROCESS")
oldsel=SELECT()
st="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"        && "Digits" of our base-36 number
SET REPROCESS TO -1
IF USED("_X_UNIQ")
   SELECT _X_UNIQ
ELSE
   * IF file doesn't exist, create it with 1 record.
   IF !FILE("_X_UNIQ.DBF")
      CREATE TABLE _X_UNIQ (IDTAG C(10), IDNUM N(13,0))
      INDEX ON IDTAG TAG IDTAG
      USE                                        && The created file will be used exclusively, so close it.
   ENDIF
   SELECT 0
   USE _X_UNIQ SHARED ORDER IDTAG                && Use file as shared regardless of SET EXCLUSIVE
   * NOTICE !!!! If you are using UNIQID version prior to 1.35, read below!!!
   
   ****!!! Following Code is used to update _x_uniq for those
   * who are upgrading from a previous version.
   * NOTICE!!!! - will not update versions prior to 1.35
   * If you have a version prior to 1.35, you will have to 
   * modify the structure of _X_UNIQ to add a 6-character field
   * named IDTAG, and index on IDTAG, TAG IDTAG. Or else, delete
   * _X_UNIQ.DBF completely if you don't mind the return value
   * resetting to 00000001.
   *
   * If you have not used UNIQID.PRG before, you can
   * safely remove all code, including this comment
   * up to and including the *END UPDATE CODE    line
   * You can also remove the U_ERROR PROCEDURE
   * at the end of this .PRG
   PRIVATE oerror, osafe
   IF FSIZE("IDTAG")<>10			                && Check for proper size of database
   	USE IN _X_UNIQ                             && Close shared _X_UNIQ
   	oerror=ON("ERROR")                         && Save current ON ERROR setting
   	ON ERROR DO U_ERROR                        && Set up own error trapping.
   	USE _X_UNIQ EXCL                           && Try for exclusive use of _X_UNIQ so we can update it.
   	ON ERROR &oerror                           && Restore original ON ERROR setting
   	IF USED("_X_UNIQ")                         && Were we successful in opening _X_UNIQ EXCLUSIVELY?
   		CREATE CURSOR _X_OLDU_Z_ (IDTAG C(6), IDNUM N(13,0))   && Create temp holdin place 
   		APPEND FROM _X_UNIQ                     && Copy data from old _X_UNIQ 
   		osafe=SET("SAFETY")                     && Save current SET SAFETY setting
   		SET SAFETY OFF                          && We don't want to bother user with those pesky dialogs.
   		SELECT _X_UNIQ                          && Close the "old" copy of database.
   		USE
   		CREATE TABLE _X_UNIQ (IDTAG C(10), IDNUM N(13,0))  && Replace with new structure
   		SET SAFETY &osafe                       && Restore SAFETY setting
   		INDEX ON IDTAG TAG IDTAG                && Create needed index.
   		APPEND FROM (DBF("_X_OLDU_Z_"))         && Fill with "old" data..
   		GO TOP
   		IF EMPTY(IDTAG)                         && Updating from pre-v1.35
   			REPLACE IDTAG WITH "DEFAUL"
   		ENDIF
   		USE IN _X_OLDU_Z_                       && Close temporary database.
   		USE                                     && Close exclusively used _X_UNIQ
   	ENDIF
		USE _X_UNIQ SHARED ORDER IDTAG             && Re-open _X_UNIQ.. if everything went ok, IDTAG should now be 10 chars.
   ENDIF
   *END UPDATE CODE
ENDIF
IF !SEEK(m.tag)                                  && If ID TAG does not exist,
  	INSERT INTO _X_UNIQ (IDTAG) VALUES (m.tag)    && Add it.
ENDIF
WAIT WINDOW "Getting ID #" NOWAIT                && Let user know what is going on in case there is some competition for access.  (Function runs 35% faster without WAIT/WAIT CLEAR)
=LOCK()                                          && Because of the REPROCESS setting, this will wait here until lock is obtained.
WAIT CLEAR
m.idnum=_x_uniq.idnum+1                          && Add 1 to counter
IF m.idnum>2821109907455                         && Just in case someone actually does roll the counter.
    m.idnum=0                                    && BTW - in single user time tests on a 486dx2/66Mhz computer, that would take almost 500 years.
ENDIF
REPLACE _x_uniq.idnum WITH m.idnum               && Update Database
UNLOCK                                           && Let someone else have a chance.
m.ret=""
*** This DO WHILE loop converts m.idnum (base-10) to a base-36 number.
DO WHILE m.idnum>0
    m.ret=SUBSTR(m.st,(m.idnum%36)+1,1)+m.ret
    m.idnum=INT(m.idnum/36)
ENDDO
SELECT (oldsel)                                  && Restore starting work area
SET REPROCESS TO (m.oldrepro)                    && Restore the SET REPROCESS setting to whatever it was.
RETURN PADL(m.ret,8,"0")                         && Return our base-36 number as a 8 character string.


PROCEDURE U_ERROR
* This procedure handles error caused by attempting to use _X_UNIQ
* exclusively for update... If you removed the updating code from 
* above, you can also remove this entire PROCEDURE
PRIVATE mans
DO WHILE .T.
 WAIT CLEAR
 WAIT WINDOW "FILE _X_UNIQ needs to be updated... However, others seem"+CHR(13)+;
	"to be using this file currently.  To re-try update, have all others"+CHR(13)+;
	"quit using this program... then press R."+CHR(13)+;
	"To skip update and quit program (not recommended) press S now." ;
	TO mans
 mans=UPPER(mans)
 DO CASE
 CASE mans="S"
 	SUSPEND
 CASE mans="R"
 	RETRY
 OTHERWISE
 	??CHR(7)
 ENDCASE
ENDDO